--[[
  * Wykkyd [ Multi-Screen ]
  * Sponsored & Supported by: The Prydonian Elders
  * Author: Ravalox Darkshire, theviper121(updates)
  * Special Thanks To: Zenimax Online Studios & Bethesda for The Elder Scrolls Online
]]--

local _addon = {}
_addon._v = {}
_addon.Version 	= "3.9"
_addon.Name			= "wykkydsMulti-Screen"
_addon.DisplayName  = "Wykkyd Multi-Screen"
_addon.savedVarsName = "wykkydsGlobalSettings"
_addon.savedVarsVersion	= 2
_addon.defaultSettings = {
	enabled = false,
	screen_rows = 1,
	screens_per_row = 3,
	screen_target_row = 1,
	screen_target_screen = 2,
	correct_bezel = false,
	manual_bezel = false,
	config_bezel = 50,
	repositon = true,
}

local baseX, baseY = nil, nil
local OriginalFunctions, OriginalUIWidth = {}, {}

local validMinMax = function( key, min, max )
	local ret = tonumber(_addon.settings[key])
	if ret < min then ret = min end
	if ret > max then ret = max end
	return ret
end

local AspectRatios = { -- Thanks to Tinuviel for this ;)
    1.33,   --4:3
    1.5,    --3:2 (old TV's)
    1.6,    --16:10
    1.78    --16:9
}

local floor = math.floor
local round = function(num, idp)
	local mult = 10^(idp or 0)
	return floor(num * mult + 0.5) / mult
end

local GetMonitorWidth = function() -- Thanks to Tinuviel for the base this was built from
    local uiWidth = floor(baseX + 0.5)
    local monitorHeight = floor(baseY + 0.5)
    local monitorWidth = uiWidth / 3
    local aspectRatio = round(monitorWidth / monitorHeight, 2)
    local correctedRatio, hasGoodRatio = 0, false

    for i, val in ipairs(AspectRatios) do
        if val == aspectRatio then hasGoodRatio = true; break; end
    end

    if (not hasGoodRatio) then
        for i, val in ipairs(AspectRatios) do
            if aspectRatio > val and val > correctedRatio then correctedRatio = val; end
        end
    else correctedRatio = aspectRatio end

    if (correctedRatio == 1.78) then correctedRatio = 1.7777777777777777777 end
    if (correctedRatio == 1.33) then correctedRatio = 1.3333333333333333333 end

    local correctedMonitorWidth = round(correctedRatio * monitorHeight, 0)
    local bezelWidth = round(uiWidth - (correctedMonitorWidth * 3))
    local bezelCorrection = 0

    if (bezelWidth > 0) then bezelCorrection = bezelWidth end

    return correctedMonitorWidth, bezelCorrection
end

local attemptedReposition = false

local AdjustScreen = function(noReload, eventCode)
	if not _addon.settings[ "enabled" ] then return end
	local screensPerRow	= validMinMax("screens_per_row", 1, 5)
	local rows 			= validMinMax("screen_rows", 1, 3)
	local targetRow		= validMinMax("screen_target_row", 1, rows)
	local targetScreen	= validMinMax("screen_target_screen", 1, screensPerRow)
	local adjustBezel	= _addon.settings[ "correct_bezel" ]
	local manualBezel	= _addon.settings[ "manual_bezel" ]
	local configBezel	= validMinMax("config_bezel", 1, 500)
	local repositon 	= _addon.settings[ "repositon" ]
	if screensPerRow * rows <= 1 then CHAT_ROUTER:AddSystemMessage( "[MultiScreen] Cannot support Multi-Screen on a single screen. Check your settings." ); return; end
	
	if baseX == nil then baseX = GuiRoot:GetWidth() end
	if baseY == nil then baseY = GuiRoot:GetHeight() end
	if not noReload then
		if baseX ~= GuiRoot:GetWidth() or baseY ~= GuiRoot:GetHeight() then
			ReloadUI()
			return
		end
	end
		
	local targetWidth 	= baseX / screensPerRow
	local targetHeight	= baseY / rows
	local resultWidth 	= targetWidth
	local gui = _G[ "wykkydMulti-ScreenGuiRoot" ]
	if not gui then 
		gui = WINDOW_MANAGER:CreateTopLevelWindow()
	end
    gui:SetClampedToScreen(true)
    gui:SetMouseEnabled(false)
    gui:SetMovable(false)
    gui:SetHidden(true)
	gui:SetDimensions( baseX, baseY )
	GuiRoot:SetParent( gui )
	GuiRoot:ClearAnchors()
	GuiRoot:SetWidth(targetWidth)
	GuiRoot:SetHeight(targetHeight)
	local offsetX = (targetScreen - 1) * targetWidth
	local offsetY = (targetRow - 1) * targetHeight
	GuiRoot:SetAnchor(TOPLEFT, gui, TOPLEFT, offsetX, offsetY)
	
	if adjustBezel then
		local correctedMonitorWidth, bezelCorrection = GetMonitorWidth()
		GuiRoot:SetWidth( correctedMonitorWidth )
		local pixelsPerBezelRow = bezelCorrection / (screensPerRow - 1)
		if manualBezel then pixelsPerBezelRow = configBezel end
		offsetX = (targetScreen - 1) * correctedMonitorWidth + (pixelsPerBezelRow * (targetScreen - 1))
		offsetY = (targetRow - 1) * targetHeight
		GuiRoot:ClearAnchors()
		GuiRoot:SetAnchor(TOPLEFT, gui, TOPLEFT, offsetX, offsetY)
	end
	if repositon then
		
		if not attemptedReposition then
			OriginalUIWidth["ZO_SharedRightPanelBackgroundLeft"] = ZO_SharedRightPanelBackgroundLeft:GetWidth()
			OriginalUIWidth["ZO_SharedStatsBackgroundBG"] = ZO_SharedStatsBackgroundBG:GetWidth()
			OriginalUIWidth["ZO_WorldMapInfoFootPrintBackgroundBG"] = ZO_WorldMapInfoFootPrintBackgroundBG:GetWidth() -- right map panel
			OriginalFunctions.ZO_InteractionManager_PopulateChatterOptions = ZO_InteractionManager.PopulateChatterOptions
			OriginalFunctions.ZO_Tooltips_SetupDynamicTooltipAnchors = ZO_Tooltips_SetupDynamicTooltipAnchors
			OriginalFunctions.GetDigToolUIKeybindPosition = GetDigToolUIKeybindPosition
			OriginalFunctions.GetDigPowerBarUIPosition = GetDigPowerBarUIPosition
			OriginalFunctions.GetRadarCountUIPosition = GetRadarCountUIPosition
			EVENT_MANAGER:RegisterForEvent(_addon.Name, EVENT_QUEST_OFFERED, function() ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0); end, false )
			EVENT_MANAGER:RegisterForEvent(_addon.Name, EVENT_QUEST_COMPLETE_DIALOG, function() ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0); end, false )
			
			OriginalUIWidth["ZO_SharedMediumLeftPanelBackgroundLeft"] = ZO_SharedMediumLeftPanelBackgroundLeft:GetWidth() -- left map and skill panel
			OriginalUIWidth["ZO_SharedWideLeftPanelBackgroundLeft"] = ZO_SharedWideLeftPanelBackgroundLeft:GetWidth() -- left inventory (character window)
			OriginalUIWidth["ZO_SharedRightBackground"] = ZO_SharedRightBackground:GetWidth() -- Used for crown store
		end
		attemptedReposition = true
	
		ZO_SharedRightPanelBackgroundLeft:SetWidth(OriginalUIWidth["ZO_SharedRightPanelBackgroundLeft"] - 385)
		ZO_SharedStatsBackgroundBG:SetWidth(OriginalUIWidth["ZO_SharedStatsBackgroundBG"] - 305)
		ZO_WorldMapInfoFootPrintBackgroundBG:SetWidth(OriginalUIWidth["ZO_WorldMapInfoFootPrintBackgroundBG"] - 600)
		ZO_SharedThinLeftPanelBackground:ClearAnchors()
		ZO_SharedThinLeftPanelBackground:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, 75, 200)
		ZO_SharedThinLeftPanelBackground:SetHeight(ZO_SharedThinLeftPanelBackground:GetHeight() + 50)
		ZO_SharedThinLeftPanelBackgroundLeft:SetWidth(337)
		
		ZO_SharedMediumLeftPanelBackground:ClearAnchors()
		ZO_SharedMediumLeftPanelBackground:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, 50, 200)
		ZO_SharedMediumLeftPanelBackgroundLeft:SetWidth(OriginalUIWidth["ZO_SharedMediumLeftPanelBackgroundLeft"] - 600)
		ZO_SharedWideLeftPanelBackgroundLeft:SetWidth(OriginalUIWidth["ZO_SharedWideLeftPanelBackgroundLeft"] - 450)
		ZO_SharedWideLeftPanelBackground:ClearAnchors()
		ZO_SharedWideLeftPanelBackground:SetAnchor(TOPLEFT, GuiRoot, TOPLEFT, 50, 200)
		
		-- Move popup windows to proper screen
		ZO_InteractWindowDivider:ClearAnchors()
		ZO_InteractWindowDivider:SetAnchor(RIGHT, GuiRoot, TOPRIGHT,  0, targetHeight * .5)
		ZO_InteractWindowDivider:SetWidth(resultWidth/2)
		ZO_InteractWindowTopBG:SetAnchor(BOTTOMRIGHT, GuiRoot, RIGHT, 0, 0)
		ZO_InteractionManager.PopulateChatterOptions = function(...) OriginalFunctions.ZO_InteractionManager_PopulateChatterOptions(...); ZO_InteractWindowBottomBG:SetAnchor(TOPRIGHT, GuiRoot, RIGHT, 0, 0); end
		ZO_Tooltips_SetupDynamicTooltipAnchors = function(a, b, c, d) OriginalFunctions.ZO_Tooltips_SetupDynamicTooltipAnchors(a, b, c, d); if (b:GetType() == 7) then a:ClearAnchors(); a:SetAnchor(LEFT, b, RIGHT, 10, 0); end end
		
		-- Fix excavation window
		GetDigToolUIKeybindPosition = function(a) 
			local tempX, tempY = OriginalFunctions.GetDigToolUIKeybindPosition(a)
			if tempX > targetWidth then
				tempX = tempX - targetWidth
			end
			return tempX, tempY
		end
		GetDigPowerBarUIPosition = function(a) 
			local tempX, tempY = OriginalFunctions.GetDigPowerBarUIPosition(a)
			if tempX > targetWidth then
				tempX = tempX - targetWidth
			end
			return tempX, tempY
		end
		GetRadarCountUIPosition = function(a) 
			local tempX, tempY = OriginalFunctions.GetRadarCountUIPosition(a)
			if tempX > targetWidth then
				tempX = tempX - targetWidth
			end
			return tempX, tempY
		end
		
		-- Fix chat window
		ZO_ChatWindowMinimize:SetHandler( "OnClicked", function(self, button)
			ZO_ChatWindow:SetClampedToScreen(false)
			ZO_ChatWindow:ClearAnchors();
			ZO_ChatWindow:SetAnchor( BOTTOMLEFT, GuiRoot, BOTTOMLEFT, 0, 0 )
			KEYBOARD_CHAT_SYSTEM:Minimize()
		end )
		ZO_ChatWindowMinBarMaximize:SetHandler( "OnClicked", function(self, button)
			ZO_ChatWindow:SetClampedToScreen(true)
			ZO_ChatWindow:ClearAnchors()
			ZO_ChatWindow:SetAnchor( BOTTOMLEFT, GuiRoot, BOTTOMLEFT, 0, 0 )
			KEYBOARD_CHAT_SYSTEM:Maximize()
		end )
		ZO_ChatWindow:SetHandler( "OnMouseDown", function(self, button)
			ZO_ChatWindow:ClearAnchors()
			ZO_ChatWindow:SetAnchor( BOTTOMLEFT, GuiRoot, BOTTOMLEFT, 0, 0 )
		end )
		
		zo_callLater(function ()
			ZO_ChatWindow:SetClampedToScreen(true)
			ZO_ChatWindow:ClearAnchors();
			ZO_ChatWindow:SetAnchor( BOTTOMLEFT, GuiRoot, BOTTOMLEFT, 0, 0 )
			KEYBOARD_CHAT_SYSTEM:Minimize()
		end, 500)
		
	else
		if attemptedReposition then ReloadUI() end
	end
end

-- Fix 3d item preview scenes
function ZO_ItemPreview_Keyboard:OnPreviewShown()
	local scene_name = SCENE_MANAGER.currentScene.name
	if (scene_name == "tradinghouse") or (scene_name == "keyboard_housing_furniture_scene") or (scene_name == "store") then
		local location = function()return GuiRoot:GetWidth()/3, .55 * GuiRoot:GetHeight() end
		local locationFragment = ZO_NormalizedPointFragment:New(location, SetFrameLocalPlayerTarget)
		SCENE_MANAGER:RemoveFragment(FRAME_TARGET_STANDARD_RIGHT_PANEL_FRAGMENT)
		SCENE_MANAGER:AddFragment(locationFragment)
	elseif (scene_name == "stables") or (scene_name == "alchemy") or (scene_name == "enchanting") or (scene_name == "smithing") then
		local location = function()return GuiRoot:GetWidth()/2.5, .6 * GuiRoot:GetHeight() end
		local locationFragment = ZO_NormalizedPointFragment:New(location, SetFrameLocalPlayerTarget)
		SCENE_MANAGER:RemoveFragment(FRAME_TARGET_STANDARD_RIGHT_PANEL_FRAGMENT)
		SCENE_MANAGER:AddFragment(locationFragment)
	end
end

-- Fix character placement in skills tab
function ZO_SkillsManager:UpdateSkillsAdvisorVisibility()
    if SKILLS_FRAGMENT:IsShowing() then
		local location = function()return GuiRoot:GetWidth()/2.5, .55 * GuiRoot:GetHeight() end
		local locationFragment = ZO_NormalizedPointFragment:New(location, SetFrameLocalPlayerTarget)
		SCENE_MANAGER:AddFragment(locationFragment)
		
		if not ZO_SKILLS_ADVISOR_SINGLETON:IsAdvancedModeSelected() or ZO_SkillsManager.showAdvisorInAdvancedMode then
			SCENE_MANAGER:AddFragment(SKILLS_ADVISOR_FRAGMENT)
		else
			SCENE_MANAGER:RemoveFragment(SKILLS_ADVISOR_FRAGMENT)
		end
        ZO_SkillsManager:UpdateKeybinds()
    end
end

-- Fix collection outfits 
ZO_OUTFIT_STYLES_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL_MEDIUM_LEFT_PANEL)
local location = function()return GuiRoot:GetWidth()/2.5, .55 * GuiRoot:GetHeight() end
local locationFragment = ZO_NormalizedPointFragment:New(location, SetFrameLocalPlayerTarget)
ZO_OUTFIT_STYLES_BOOK_SCENE:AddFragment(locationFragment)
ZO_OUTFIT_STYLES_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)


local location = function()return GuiRoot:GetWidth()/2.8, .55 * GuiRoot:GetHeight() end
local locationFragment = ZO_NormalizedPointFragment:New(location, SetFrameLocalPlayerTarget)

-- Fix companion equipment scene
COMPANION_CHARACTER_KEYBOARD_SCENE:RemoveFragment(FRAME_INTERACTION_STANDARD_RIGHT_PANEL_MEDIUM_LEFT_PANEL_FRAGMENT)
COMPANION_CHARACTER_KEYBOARD_SCENE:AddFragment(locationFragment)
COMPANION_CHARACTER_KEYBOARD_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix companion skills scene
COMPANION_SKILLS_KEYBOARD_SCENE:RemoveFragment(FRAME_INTERACTION_STANDARD_RIGHT_PANEL_MEDIUM_LEFT_PANEL_FRAGMENT)
COMPANION_SKILLS_KEYBOARD_SCENE:AddFragment(locationFragment)
COMPANION_SKILLS_KEYBOARD_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix companion collection scene
COMPANION_COLLECTION_BOOK_KEYBOARD_SCENE:RemoveFragment(FRAME_INTERACTION_STANDARD_RIGHT_PANEL_MEDIUM_LEFT_PANEL_FRAGMENT)
COMPANION_COLLECTION_BOOK_KEYBOARD_SCENE:AddFragment(locationFragment)
COMPANION_COLLECTION_BOOK_KEYBOARD_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix collections main tab
COLLECTIONS_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
COLLECTIONS_BOOK_SCENE:AddFragment(locationFragment)
COLLECTIONS_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix collections housing tab
HOUSING_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
HOUSING_BOOK_SCENE:AddFragment(locationFragment)
HOUSING_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix collections DLC book (stories) tab
DLC_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
DLC_BOOK_SCENE:AddFragment(locationFragment)
DLC_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix collections item sets tab
ITEM_SETS_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL_MEDIUM_LEFT_PANEL)
ITEM_SETS_BOOK_SCENE:AddFragment(locationFragment)
ITEM_SETS_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix collections tribute book tab
TRIBUTE_PATRON_BOOK_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
TRIBUTE_PATRON_BOOK_SCENE:AddFragment(locationFragment)
TRIBUTE_PATRON_BOOK_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal quests tab
QUEST_JOURNAL_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
QUEST_JOURNAL_SCENE:AddFragment(locationFragment)
QUEST_JOURNAL_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal antiquities tab
local antiquityJournalScene = SCENE_MANAGER:GetScene("antiquityJournalKeyboard")
antiquityJournalScene:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
antiquityJournalScene:AddFragment(locationFragment)
antiquityJournalScene:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal antiquitiesLore tab
local antiquityLoreScene = SCENE_MANAGER:GetScene("antiquityLoreKeyboard")
antiquityLoreScene:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
antiquityLoreScene:AddFragment(locationFragment)
antiquityLoreScene:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal lore Library tab
LORE_LIBRARY_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
LORE_LIBRARY_SCENE:AddFragment(locationFragment)
LORE_LIBRARY_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal achievements tab
local achievementsScene = SCENE_MANAGER:GetScene("achievements")
achievementsScene:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
achievementsScene:AddFragment(locationFragment)
achievementsScene:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix journal leaderboard tab
LEADERBOARDS_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
LEADERBOARDS_SCENE:AddFragment(locationFragment)
LEADERBOARDS_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Group tab
KEYBOARD_GROUP_MENU_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
KEYBOARD_GROUP_MENU_SCENE:AddFragment(locationFragment)
KEYBOARD_GROUP_MENU_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Friend tab
FRIENDS_LIST_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
FRIENDS_LIST_SCENE:AddFragment(locationFragment)
FRIENDS_LIST_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Friend Ignore tab
IGNORE_LIST_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
IGNORE_LIST_SCENE:AddFragment(locationFragment)
IGNORE_LIST_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Home tab
GUILD_HOME_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
GUILD_HOME_SCENE:AddFragment(locationFragment)
GUILD_HOME_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Ranks tab
GUILD_RANKS_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
GUILD_RANKS_SCENE:AddFragment(locationFragment)
GUILD_RANKS_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Roster tab
GUILD_ROSTER_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
GUILD_ROSTER_SCENE:AddFragment(locationFragment)
GUILD_ROSTER_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild History tab
--GUILD_HISTORY_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
--GUILD_HISTORY_SCENE:AddFragment(locationFragment)
--GUILD_HISTORY_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Heraldry tab
GUILD_HERALDRY_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
GUILD_HERALDRY_SCENE:AddFragment(locationFragment)
GUILD_HERALDRY_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Finder tab
KEYBOARD_GUILD_RECRUITMENT_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
KEYBOARD_GUILD_RECRUITMENT_SCENE:AddFragment(locationFragment)
KEYBOARD_GUILD_RECRUITMENT_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Browser tab
KEYBOARD_GUILD_BROWSER_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
KEYBOARD_GUILD_BROWSER_SCENE:AddFragment(locationFragment)
KEYBOARD_GUILD_BROWSER_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Info tab
KEYBOARD_LINK_GUILD_INFO_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
KEYBOARD_LINK_GUILD_INFO_SCENE:AddFragment(locationFragment)
KEYBOARD_LINK_GUILD_INFO_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Guild Create tab
GUILD_CREATE_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
GUILD_CREATE_SCENE:AddFragment(locationFragment)
GUILD_CREATE_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Campaign Browser tab
CAMPAIGN_BROWSER_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
CAMPAIGN_BROWSER_SCENE:AddFragment(locationFragment)
CAMPAIGN_BROWSER_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Campaign Overview tab
CAMPAIGN_OVERVIEW_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
CAMPAIGN_OVERVIEW_SCENE:AddFragment(locationFragment)
CAMPAIGN_OVERVIEW_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Mail Inbox tab
MAIL_INBOX_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
MAIL_INBOX_SCENE:AddFragment(locationFragment)
MAIL_INBOX_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Mail Send tab
MAIL_SEND_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
MAIL_SEND_SCENE:AddFragment(locationFragment)
MAIL_SEND_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Notifications tab
NOTIFICATIONS_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
NOTIFICATIONS_SCENE:AddFragment(locationFragment)
NOTIFICATIONS_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Help tab
HELP_CUSTOMER_SUPPORT_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
HELP_CUSTOMER_SUPPORT_SCENE:AddFragment(locationFragment)
HELP_CUSTOMER_SUPPORT_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Help Tutorials tab
local helpTutorialsScene = SCENE_MANAGER:GetScene("helpTutorials")
helpTutorialsScene:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
helpTutorialsScene:AddFragment(locationFragment)
helpTutorialsScene:AddFragment(FRAME_PLAYER_FRAGMENT)

-- Fix Emotes tab
HELP_EMOTES_SCENE:RemoveFragmentGroup(FRAGMENT_GROUP.FRAME_TARGET_STANDARD_RIGHT_PANEL)
HELP_EMOTES_SCENE:AddFragment(locationFragment)
HELP_EMOTES_SCENE:AddFragment(FRAME_PLAYER_FRAGMENT)

local eventRegistered = false

local prepAdjustScreen = function(noReload)
	if _addon.settings[ "enabled" ] then
		AdjustScreen(noReload or false)
		if not eventRegistered then
			EVENT_MANAGER:RegisterForEvent(_addon.Name, EVENT_SCREEN_RESIZED, function(eventCode) AdjustScreen(false, eventCode) end)
			eventRegistered = true
		end
	else
		if eventRegistered then
			EVENT_MANAGER:RegisterForEvent(_addon.Name, EVENT_SCREEN_RESIZED)
			ReloadUI()
		end
	end
end

_addon.LoadSettingsMenu = function()
	local panelData = {
		type = "panel",
		name = _addon.DisplayName,
		displayName = _addon.DisplayName,
		author = "Exodus Group, theviper121(updates)",
		version = _addon.Version,
		registerForRefresh = true,
		registerForDefaults = true,
	}
	local optionsTable = {
		{
			type = "description",
			text = "Multi-Screen support requires the use of 3rd party software and/or hardware. The most common being AMD Eyefinity and nVidia Surround.",
		},{
			type = "description",
			text = "Once you have the game drawing across multiple screens at once this addon becomes useful to you. If using a single screen, or a duplicated screen across multiple monitors, then this addon is no benefit to you!",
		},{
			type = "checkbox",
			name = "Multi-Screen Support",
			getFunc = function() return _addon.settings.enabled end,
			setFunc = function(value) _addon.settings.enabled = value; ReloadUI() end,
			warning = "Reloads your UI when setting is turned off",
            default = false,
		},{
			type = "slider",
			name = "Rows of Monitors?",
			getFunc = function() return _addon.settings.screen_rows end,
			setFunc = function(value) _addon.settings.screen_rows = value; prepAdjustScreen(true) end,
			min = 1,
			max = 3,
			step = 1,
            default = 1,
		},{
			type = "slider",
			name = "Monitors per Row?",
			getFunc = function() return _addon.settings.screens_per_row end,
			setFunc = function(value) _addon.settings.screens_per_row = value; prepAdjustScreen(true) end,
			min = 1,
			max = 5,
			step = 1,
            default = 3,
		},{
			type = "slider",
			name = "Row Containing UI?",
			getFunc = function() return _addon.settings.screen_target_row end,
			setFunc = function(value) _addon.settings.screen_target_row = value; prepAdjustScreen(true) end,
			min = 1,
			max = 3,
			step = 1,
            default = 1,
		},{
			type = "slider",
			name = "Which Monitor in that Row Contains UI?",
			getFunc = function() return _addon.settings.screen_target_screen end,
			setFunc = function(value) _addon.settings.screen_target_screen = value; prepAdjustScreen(true) end,
			min = 1,
			max = 5,
			step = 1,
            default = 2,
		},{
			type = "checkbox",
			name = "Bezel Correction",
			getFunc = function() return _addon.settings.correct_bezel end,
			setFunc = function(value) _addon.settings.correct_bezel = value; prepAdjustScreen(true) end,
            default = false,
		},{
			type = "checkbox",
			name = "Manual Bezel Correction",
			getFunc = function() return _addon.settings.manual_bezel end,
			setFunc = function(value) _addon.settings.manual_bezel = value; prepAdjustScreen(true) end,
            default = false,
		},{
			type = "slider",
			name = "Manual Bezel Size",
			tooltip = "Only works if Manual Bezel Correction is on!",
			getFunc = function() return _addon.settings.config_bezel end,
			setFunc = function(value) _addon.settings.config_bezel = value; prepAdjustScreen(true) end,
			min = 1,
			max = 500,
			step = 1,
            default = 50,
		},{
			type = "checkbox",
			name = "Element Repositoning",
			tooltip = "Only disable if this gives you issues!",
			warning = "Reloads your UI when setting is turned off",
			getFunc = function() return _addon.settings.repositon end,
			setFunc = function(value) _addon.settings.repositon = value; prepAdjustScreen() end,
            default = true,
		}
	}

	local lam = LAM2 or LibAddonMenu2
	lam:RegisterAddonPanel("wykkydsMulti-Screen_LAM", panelData)
	lam:RegisterOptionControls("wykkydsMulti-Screen_LAM", optionsTable)
end

local addon_loaded = false
_addon.loaded = function(eventCode, addonName)
	if _addon.Name ~= addonName then
        return
    end
    EVENT_MANAGER:UnregisterForEvent(addonName, EVENT_ADD_ON_LOADED)
	addon_loaded = true
	if not _addon.settings then
		_addon.settings = {}
		_addon.settings = ZO_SavedVars:NewAccountWide(_addon.savedVarsName, _addon.savedVarsVersion, nil, _addon.defaultSettings)
	end
	_addon.LoadSettingsMenu()
	prepAdjustScreen()
end

EVENT_MANAGER:RegisterForEvent(_addon.Name, EVENT_ADD_ON_LOADED, _addon.loaded)